/********************************************************************
 FileName:      miwi_utilities.h
 Dependencies:  See INCLUDES section
********************************************************************/

#ifndef MIWI_UTILITIES_H
    #define MIWI_UTILITIES_H

    /************************ HEADERS **********************************/

    /********************** DEFINITIONS*********************************/
    /* MIWI_STATES is used by the MiWI_AppTasks SWITCH_CASE to progress
     * through the various states during MiWi operation.  See MiWi_ AppTasks
     * for the definitions of each state.                              */

    typedef enum
    {
        MIWI_STATE_INIT=0,
        MIWI_STATE_CREATE,
        MIWI_STATE_RETRY_CREATE,
        MIWI_STATE_WAIT_4_CONNECTS,
        MIWI_STATE_WAIT_2_JOIN,
        MIWI_STATE_JOIN,                
        MIWI_STATE_RETRY_JOIN,
        MIWI_STATE_ACTIVE,
        MIWI_STATE_IDLE
    }MIWI_STATES;
    
    typedef enum
    {
        MIWI_NET_STATE_OFF=0,           // MiWi is not on
        MIWI_NET_STATE_INIT,            // MiWi initialization has started
        MIWI_NET_STATE_FAILED,          // MiWi failed to start
        MIWI_NET_STATE_NOCONNECT,       // MiWi init complete but no connections
        MIWI_NET_STATE_FORMING,         // MiWi has at least one connection
        MIWI_NET_STATE_FORMED           // MiWi connections >= expected connections
    }MIWI_NET_STATES;
    
    /* this is the data structure for all MiWi data */
    typedef struct
    {
        MIWI_STATES appState;           // this is the MiWi app state
        MIWI_NET_STATES netState,       // this is the current MiWi net state, only applies to coordinators
                    netState_OLD;       // previous net state
        bool        MiWiConnection,     // indicates if the coordinator and one end device are connected
                    NetworkFormed;      // indicates if all the expected end devices are connected
        uint8_t     myChannel;          // the channel number this device uses
        uint8_t     mySlaveAddress;     // this is the app number, 1 through X
        uint8_t     MiWiConnectionCount;// the current number of end devices connected, has no meaning for end devices
        uint8_t     ExpectedCount;      // how many connections are expected, load from EEPROM
    }MIWI_DATA;

    #define cMyChannel      24

    /*******************************************************************/
    /******************** VARs and Prototype Functions *****************/
    /*******************************************************************/
    extern MIWI_DATA miwiData;
    
    bool MiWi_NetworkCommon(void);
    MIWI_NET_STATES MiWi_CreatNetwork(void);
    bool MiWi_JoinNetwork(void);
    void MiWi_ChannelScan(uint8_t myMiWiChannel);
    uint8_t MiWi_CountConnections(void);
    void MiWi_AppTasks(void);
   
#endif  //MIWI_UTILITIES_H
